from __future__ import annotations

from collections.abc import Iterator
from typing import Any

from .data import InMemoryData


class InMemoryCacheRepository:
    _cache: dict[str, InMemoryData] = {}

    def __init__(self):
        pass

    def get_value(self, key: str) -> Any:
        return self._get_data(key).value

    def get_last_modification_time(self, key: str):
        return self._get_data(key).last_modification_time

    def _get_data(self, key: str) -> InMemoryData:
        return self._cache[key]

    def set_value(self, key: str, obj: Any) -> None:
        data = InMemoryData.create_data(obj)
        self._cache[key] = data

    def has(self, key: str) -> bool:
        return key in self._cache

    def remove(self, key: str) -> None:
        assert self.has(key), f'{key} does not exist.'
        del self._cache[key]

    def empty(self) -> bool:
        return not self._cache

    def clear(self) -> None:
        self._cache.clear()

    def get_gen(self) -> Iterator[tuple[str, Any]]:
        for key, data in self._cache.items():
            yield key, data.value

    @property
    def size(self) -> int:
        return len(self._cache)
